/*******************************************************/
/* 						       */
/*           Magma code accompanying Section 3         */ 
/*              				       */
/*                    by Dino Festi                    */
/*						       */
/*******************************************************/


// First we check that the equation (2.10) can be rewritten as in (3.1)
print "First we check that the equation (2.10) can be rewritten as in (3.1)";
IP2<x,y,z>:=ProjectivePlane(Rationals());
f6:=4*x*y^2*z*(x+z)^2+(x*(y+z)^2+y*(x+z)^2)*(x*(-y+z)^2+y*(z-x)^2);
assert f6 eq 4*x*y^2*z*(x-z)^2+(x+y)^2*(x*y+z^2)^2;



// Proof of Lemma 3.1
print "Verifying Lemma 3.1";
IP2<x,y,z>:=ProjectivePlane(Rationals());
f6:=4*x*y^2*z*(x+z)^2+(x*(y+z)^2+y*(x+z)^2)*(x*(-y+z)^2+y*(z-x)^2);
B:=Curve(IP2,f6);
assert IsReduced(B);
assert IsIrreducible(B);
assert Genus(B) eq 2;




SPts:=SingularPoints(B);
assert not HasSingularPointsOverExtension(B);
for P in SPts do
  assert Multiplicity(B!P) eq 2;
end for;

P1:=B![1,-1,1];
P2:=B![1,1,-1];
P3:=B![1,0,0];
P4:=B![0,1,0];
P5:=B![0,0,1];

print "Lemma 3.1 is confirmed.";
print "**************************";
print "";
// End of Proof of Lemma 3.1


// Proof of Proposition 3.4
print "Verifying Proposition 3.4";
p:=31;
Fp:=GF(p);
  Rt<t>:=PolynomialRing(Rationals());
  IP2<x,y,z>:=ProjectivePlane(Fp);
  f6:=4*x*y^2*z*(x+z)^2+(x*(y+z)^2+y*(x+z)^2)*(x*(-y+z)^2+y*(z-x)^2);
  B:=Curve(IP2,f6);
WP1,WPE:=WeilPolynomialOfDegree2K3Surface(f6);
WP1:=Rt!WP1; WPE:=Rt!WPE;
FWPE:=Factorisation(WPE);
// One can look at the factorisations below to check that p is of good reduction
assert FWPE[1][2] eq 14;
// One can look at the factorisations below to check that the Picard number of S_p is 20
FWP1:= Factorisation(WP1);
assert FWPE[1][2]+FWP1[1][2] eq 20;
WP1T:=Rt!Factorisation(WP1)[2][1];
WP2T:=Rt!(Numerator(Evaluate(WP1T,1/t)));
// We compute the discriminant up to squares
d1,_:=SquarefreeFactorization(Integers()!(p*Evaluate(WP2T,1/p)));


p:=71;
Fp:=GF(p);
  Rt<t>:=PolynomialRing(Rationals());
  IP2<x,y,z>:=ProjectivePlane(Fp);
  f6:=4*x*y^2*z*(x+z)^2+(x*(y+z)^2+y*(x+z)^2)*(x*(-y+z)^2+y*(z-x)^2);
  B:=Curve(IP2,f6);
WP1,WPE:=WeilPolynomialOfDegree2K3Surface(f6);
WP1:=Rt!WP1; WPE:=Rt!WPE;
FWPE:=Factorisation(WPE);
// One can look at the factorisations below to check that p is of good reduction
assert Factorisation(WPE)[1][2] eq 14;
// One can look at the factorisations below to check that the Picard number of S_p is 20
FWP1:= Factorisation(WP1);
assert FWPE[1][2]+FWP1[1][2] eq 20;
WP1T:=Rt!Factorisation(WP1)[2][1];
WP2T:=Rt!(Numerator(Evaluate(WP1T,1/t)));
// We compute the discriminant up to squares
d2,_:=SquarefreeFactorization(Integers()!(p*Evaluate(WP2T,1/p)));


// We check that the discriminants so computed are not equivalent up to squares
assert d1 ne d2;

print "Proposition 3.4 is confirmed.";

// End of Proof 3.4

print "**************************";
print "";
// We define the divisors listed in Sigma and we check that the equations give indeed divisors of X_DY
print "We define the divisors listed in Sigma and we check that the equations give indeed divisors of X_DY";
K1<a>:=NumberField(Polynomial([-5,0,1]));
IP<x,y,z,w>:=ProjectiveSpace(K1,[1,1,1,3]);

t1:=(-1+a)/2;
t2:=(-1-a)/2;


ls:=[
[x, w-y*z^2],
[x-z,w-z*(y+z)^2],
[y+z/2*(3-a),w-(a-3)/2*z*(x^2-x*z+z^2)],
[y+z/2*(3+a),w-(-a-3)/2*z*(x^2-x*z+z^2)],
[y, w-x*z^2],
[y+z, w-z*(x-z)*(x+z)],
[z,w-x*y*(x+y)],
[x^2+y*z, x*y^2 - y^2*z - x*z^2 - y*z^2 - w],
[x^2+t1*(x*y+x*z)+y*z, x*y^2 + 1/2*(a + 5)*x*y*z + 1/2*(3*a + 5)*y^2*z + 1/2*(a + 3)*x*z^2 + 1/2*(3*a + 5)*y*z^2 + 1/2*(a + 
    3)*w],
[x^2+t2*(x*y+x*z)+y*z, x*y^2 + 1/2*(-a + 5)*x*y*z + 1/2*(-3*a + 5)*y^2*z + 1/2*(-a + 3)*x*z^2 + 1/2*(-3*a + 5)*y*z^2 + 1/2*(-a +
    3)*w]

];

Ls:=[Curve(IP,l) : l in ls ];

f6:=4*x*y^2*z*(x+z)^2+(x*(y+z)^2+y*(x+z)^2)*(x*(-y+z)^2+y*(z-x)^2);
X:=Scheme(IP,w^2-f6);

for L in Ls do
  assert IsSubscheme(L,X);
  assert Dimension(L) eq 1;
end for;

// We checked that all the curves defined are indeed divisors of X_DY
print "We checked that all the curves defined are indeed divisors of X_DY";


// Proof of Proposition 3.6
print "**************************";
print "";
print "Verifying Proposition 3.6";

/*  The intersection numbers between all of the curves in Sigma  come from the intersection matrix computed in the file definitions_and_basics.m  .
The code used to compute that intersection matrix is in the file IntersectionMatrixOnDegree2K3.m 
Both files are in the same directory containing the present file. */

// End of Proof of Proposition 3.6
print "Proposition 3.6 is confirmed.";



// Proof of Remark 3.7
print "**************************";
print "";
print "Verifying Remark 3.7";

SP:=[[1,-1,1,0],[1,1,-1,0],[1,0,0,0],[0,1,0,0],[0,0,1,0]];
SP:=[IP!P : P in SP];


// We define a function to compute the intersection number of the divisors in Sigma
// When the intersection number between one of the divisor L_i or C_j and one of the exceptional divisors E_{k,l}
// a message is returned and that intersection number should be handled analytically:
// it means that the image on IP^2 of the divisor is tangent to B at the singular point P_k lying under E_{k,l},
// and therefore a more careful analysis to determine which exceptional divisor above P_k actually intersects the divisor. See below. 


function GramMatrixD(List,SP)

m:=#List;
M:=ScalarMatrix(Integers(),m,-2);

for i in [1..m] do
    for j in [1..i] do
        i,j;
	Int:=List[i] meet List[j];
        if Dimension(Int) eq 1 then
	    M[i,j]:=-2;
	else
	    M[i,j]:=Degree(Int);
	    SInt:=[P : P in SP | P in Int];
            SIntm:=[Multiplicity(Int!P) : P in SInt];
	    if SIntm ne [] then
		if SIntm eq [2] then
		  print "Look at this!!!!";
		end if;
              SInt; SIntm; M[i,j];
	      M[i,j]:=M[i,j]-&+SIntm;
	    end if;
	end if;
	M[j,i]:=M[i,j];
    end for;
end for;

return M;

end function;

print "Gram matrix:";
print GramMatrixD(Ls,SP);


// We study the intersections of the divisors coming from tangent curves and the exceptional divisors
print "**************************";
print "";
print "We study the intersections of the divisors coming from tangent curves and the exceptional divisors.";
K1<a>:=NumberField(Polynomial([-5,0,1]));
P2<x,y,z>:=ProjectivePlane(K1);
f6:=4*x*y^2*z*(x+z)^2+(x*(y+z)^2+y*(x+z)^2)*(x*(-y+z)^2+y*(z-x)^2);
B:=Curve(P2,f6);
w:=1;
t1:=(-1+a)/2;
t2:=(-1-a)/2;

ls:=[
[x, w-y*z^2],
[x-z,w-z*(y+z)^2],
[y+z/2*(3-a),w-(a-3)/2*z*(x^2-x*z+z^2)],
[y+z/2*(3+a),w-(-a-3)/2*z*(x^2-x*z+z^2)],
[y, w-x*z^2],
[y+z, w-z*(x-z)*(x+z)],
[z,w-x*y*(x+y)],
[x^2+y*z, x*y^2 - y^2*z - x*z^2 - y*z^2 - w],
[x^2+t1*(x*y+x*z)+y*z, x*y^2 + 1/2*(a + 5)*x*y*z + 1/2*(3*a + 5)*y^2*z + 1/2*(a + 3)*x*z^2 + 1/2*(3*a + 5)*y*z^2 + 1/2*(a + 
    3)*w],
[x^2+t2*(x*y+x*z)+y*z, x*y^2 + 1/2*(-a + 5)*x*y*z + 1/2*(-3*a + 5)*y^2*z + 1/2*(-a + 3)*x*z^2 + 1/2*(-3*a + 5)*y*z^2 + 1/2*(-a +
    3)*w]
];

ls:=[Curve(P2,L[1]) : L in ls];

SB:=[[1,-1,1],[1,1,-1],[1,0,0],[0,1,0],[0,0,1]];
SB:=[P2!P : P in SB];


for L in ls do
  for P in SB do
    if P in L then
       if L in IrreducibleComponents(TangentCone(B,P)) then
	print "The ", L, "is in the tangent cone of B at", P;
       else
	print "The ", L, "is NOT in the tangent cone of B at", P;
       end if;
       print "********";
       print " ";
    end if;
  end for; 
end for;

// We verified which curves in Ls are contained in the tangent cone of the singular points they pass through.
print "We verified which curves in Ls are contained in the tangent cone of the singular points they pass through.";
print "**************************";
print "";
// We check that all the combinations giving a lattice with rank equal to or less than 20 give the quantities in the statement.
print "We check that all the combinations giving a lattice with rank equal to or less than 20 give the quantities in the statement.";

for i in CartesianPower([0,1],14) do

  i1:=i[1];
  i2:=i[2];
  i3:=i[3];
  i4:=i[4];
  i5:=i[5];
  i6:=i[6];
  i7:=i[7];
  i8:=i[8];
  i9:=i[9];
  i10:=i[10];
  i11:=i[11];
  i12:=i[12];
  i13:=i[13];
  i14:=i[14];
  A:=
  [
[2,	1,	1,	1,	1,	1,	1,	1,	2,	2,	2,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0],
[1,	-2,	0,	1,	1,	0,	1,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	1,	0,	0,	1,	0],
[1,	0,	-2,	0,	0,	1,	0,	0,	0,	0,	0,	0,	1,	0,	0,	0,	0,	0,	0,	1,	0,	0,	0,	0,	0],
[1,	1,	0,	-2,	0,	0,	0,	0,	1,	0,	2,	0,	0,	0,	0,	1,	0,	0,	0,	0,	0,	0,	0,	0,	0],
[1,	1,	0,	0,	-2,	0,	0,	0,	1,	2,	0,	0,	0,	0,	0,	i1,	0,	0,	1-i1,	0,	0,	0,	0,	0,	0],
[1,	0,	1,	0,	0,	-2,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	1,	0,	0,	0,	0,	0,	0,	i2,	1-i2],
[1,	1,	0,	0,	0,	0,	-2,	0,	0,	0,	0,	1,	0,	0,	1,	i3,	0,	0,	1-i3,	0,	0,	0,	0,	0,	0],
[1,	0,	0,	0,	0,	0,	0,	-2,	1,	0,	0,	0,	0,	0,	0,	i4,	0,	0,	1-i4,	i5,	0,	0,	1-i5,	0,	0],
[2,	0,	0,	1,	1,	0,	0,	1,	-2,	0,	0,	i6,	0,	1-i6,	1,	0,	0,	0,	0,	i7,	0,	0,	1-i7,	i8,	1-i8],
[2,	0,	0,	0,	2,	0,	0,	0,	0,	-2,	0,	i9,	0,	1-i9,	1,	0,	0,	0,	0,	i10,	0,	0,	1-i10,	i11,	1-i11],
[2,	0,	0,	2,	0,	0,	0,	0,	0,	0,	-2,	i12,	0,	1-i12,	1,	0,	0,	0,	0,	i13,	0,	0,	1-i13,	i14,	1-i14],
[0,	0,	0,	0,	0,	0,	1,	0,	i6,	i9,	i12,	-2,	1,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0],
[0,	0,	1,	0,	0,	0,	0,	0,	0,	0,	0,	1,	-2,	1,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0],
[0,	0,	0,	0,	0,	0,	0,	0,	1-i6,	1-i9,	1-i12,	0,	1,	-2,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0],
[0,	0,	0,	0,	0,	0,	1,	0,	1,	1,	1,	0,	0,	0,	-2,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0],
[0,	0,	0,	1,	i1,	0,	i3,	i4,	0,	0,	0,	0,	0,	0,	0,	-2,	1,	0,	0,	0,	0,	0,	0,	0,	0],
[0,	0,	0,	0,	0,	1,	0,	0,	0,	0,	0,	0,	0,	0,	0,	1,	-2,	1,	0,	0,	0,	0,	0,	0,	0],
[0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	1,	-2,	1,	0,	0,	0,	0,	0,	0],
[0,	0,	0,	0,	1-i1,	0,	1-i3,	1-i4,	0,	0,	0,	0,	0,	0,	0,	0,	0,	1,	-2,	0,	0,	0,	0,	0,	0],
[0,	0,	1,	0,	0,	0,	0,	i5,	i7,	i10,	i13,	0,	0,	0,	0,	0,	0,	0,	0,	-2,	1,	0,	0,	0,	0],
[0,	1,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	1,	-2,	1,	0,	0,	0],
[0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	1,	-2,	1,	0,	0],
[0,	0,	0,	0,	0,	0,	0,	1-i5,	1-i7,	1-i10,	1-i13,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	1,	-2,	0,	0],
[0,	1,	0,	0,	0,	i2,	0,	0,	i8,	i11,	i14,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	-2,	1],
[0,	0,	0,	0,	0,	1-i2,	0,	0,	1-i8,	1-i11,	1-i14,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	1,	-2]
];

A:=Matrix(A);
assert IsSymmetric(A);
rk:=Rank(A);
if rk le 20 then
// i;
L0:=Lattice(A);
B0:=Basis(L0);
Sol0:=Solution(A,[Vector(b) : b in B0]);
M0:=Matrix(Sol0);
Gram0:=M0*A*Transpose(M0);
print "The rank of the lattice is ", Rank(Gram0);
d0:=Determinant(Gram0);
print "The discriminant of the lattice is ", d0, ". Its Factorisation is ", Factorisation(d0);
L:=LatticeWithGram(Gram0: CheckPositive:= false);
AL,_,_:=DualQuotient(L);
print "Its discriminant group is an ", AL;
print "********";
end if;

end for;



// End of Proof of Remark 3.7
print "Remark 3.7 is confirmed.";
print "**************************";
print "";


// Proof of Theorem 3.8
print "Verifying Theorem 3.8";

// We define the Gram matrix and we check it has the correct invariants
print "We define the Gram matrix Gram0 and we check it has the correct invariants.";
K1<a>:=NumberField(Polynomial([-5,0,1]));
IP<x,y,z,w>:=ProjectiveSpace(K1,[1,1,1,3]);

i:=<1, 1, 0, 1, 1, 1, 1, 0, 0, 1, 1, 0, 1, 1>;
i1:=i[1];
  i2:=i[2];
  i3:=i[3];
  i4:=i[4];
  i5:=i[5];
  i6:=i[6];
  i7:=i[7];
  i8:=i[8];
  i9:=i[9];
  i10:=i[10];
  i11:=i[11];
  i12:=i[12];
  i13:=i[13];
  i14:=i[14];
A:=
  [
[2,	1,	1,	1,	1,	1,	1,	1,	2,	2,	2,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0],
[1,	-2,	0,	1,	1,	0,	1,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	1,	0,	0,	1,	0],
[1,	0,	-2,	0,	0,	1,	0,	0,	0,	0,	0,	0,	1,	0,	0,	0,	0,	0,	0,	1,	0,	0,	0,	0,	0],
[1,	1,	0,	-2,	0,	0,	0,	0,	1,	0,	2,	0,	0,	0,	0,	1,	0,	0,	0,	0,	0,	0,	0,	0,	0],
[1,	1,	0,	0,	-2,	0,	0,	0,	1,	2,	0,	0,	0,	0,	0,	i1,	0,	0,	1-i1,	0,	0,	0,	0,	0,	0],
[1,	0,	1,	0,	0,	-2,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	1,	0,	0,	0,	0,	0,	0,	i2,	1-i2],
[1,	1,	0,	0,	0,	0,	-2,	0,	0,	0,	0,	1,	0,	0,	1,	i3,	0,	0,	1-i3,	0,	0,	0,	0,	0,	0],
[1,	0,	0,	0,	0,	0,	0,	-2,	1,	0,	0,	0,	0,	0,	0,	i4,	0,	0,	1-i4,	i5,	0,	0,	1-i5,	0,	0],
[2,	0,	0,	1,	1,	0,	0,	1,	-2,	0,	0,	i6,	0,	1-i6,	1,	0,	0,	0,	0,	i7,	0,	0,	1-i7,	i8,	1-i8],
[2,	0,	0,	0,	2,	0,	0,	0,	0,	-2,	0,	i9,	0,	1-i9,	1,	0,	0,	0,	0,	i10,	0,	0,	1-i10,	i11,	1-i11],
[2,	0,	0,	2,	0,	0,	0,	0,	0,	0,	-2,	i12,	0,	1-i12,	1,	0,	0,	0,	0,	i13,	0,	0,	1-i13,	i14,	1-i14],
[0,	0,	0,	0,	0,	0,	1,	0,	i6,	i9,	i12,	-2,	1,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0],
[0,	0,	1,	0,	0,	0,	0,	0,	0,	0,	0,	1,	-2,	1,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0],
[0,	0,	0,	0,	0,	0,	0,	0,	1-i6,	1-i9,	1-i12,	0,	1,	-2,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0],
[0,	0,	0,	0,	0,	0,	1,	0,	1,	1,	1,	0,	0,	0,	-2,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0],
[0,	0,	0,	1,	i1,	0,	i3,	i4,	0,	0,	0,	0,	0,	0,	0,	-2,	1,	0,	0,	0,	0,	0,	0,	0,	0],
[0,	0,	0,	0,	0,	1,	0,	0,	0,	0,	0,	0,	0,	0,	0,	1,	-2,	1,	0,	0,	0,	0,	0,	0,	0],
[0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	1,	-2,	1,	0,	0,	0,	0,	0,	0],
[0,	0,	0,	0,	1-i1,	0,	1-i3,	1-i4,	0,	0,	0,	0,	0,	0,	0,	0,	0,	1,	-2,	0,	0,	0,	0,	0,	0],
[0,	0,	1,	0,	0,	0,	0,	i5,	i7,	i10,	i13,	0,	0,	0,	0,	0,	0,	0,	0,	-2,	1,	0,	0,	0,	0],
[0,	1,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	1,	-2,	1,	0,	0,	0],
[0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	1,	-2,	1,	0,	0],
[0,	0,	0,	0,	0,	0,	0,	1-i5,	1-i7,	1-i10,	1-i13,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	1,	-2,	0,	0],
[0,	1,	0,	0,	0,	i2,	0,	0,	i8,	i11,	i14,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	-2,	1],
[0,	0,	0,	0,	0,	1-i2,	0,	0,	1-i8,	1-i11,	1-i14,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	0,	1,	-2]
];
A:=Matrix(A);
L0:=Lattice(A);
B0:=Basis(L0);
Sol0:=Solution(A,[Vector(b) : b in B0]);
M0:=Matrix(Sol0);
Gram0:=M0*A*Transpose(M0);
print Gram0;
assert Rank(Gram0) eq 19;
print "Gram matrix Gram0 has rank 19.";
assert Determinant(Gram0) eq 24;
print "Gram matrix Gram0 determinant 24.";
L:=LatticeWithGram(Gram0: CheckPositive:= false);
AL,DL,phi:=DualQuotient(L);


// We define a function that computes the intersection pairing of two given vectors and an intersection matrix

function QuadInt(v,w,M)

v1:=ChangeRing(Matrix(v),Integers());
w1:=ChangeRing(Matrix(w),Integers());

return (v1*M*Transpose(w1))[1,1];

end function;

// We compute the set Lambda2

F2:=GF(2);
V2:=VectorSpace(F2,19);
A2:=ChangeRing(Gram0,F2);
k2:=sub<V2 | Kernel(A2)>;

Lambda2:=[v : v in k2 | QuadInt(v,v,Gram0) mod 8 eq 0];


// We define the matrix representing the Galois action on Lambda

G:=ScalarMatrix(25,1);
G[4,4]:=0; G[5,5]:=0; G[4,5]:=1; G[5,4]:=1;
G[10,10]:=0; G[11,11]:=0; G[10,11]:=1; G[11,10]:=1;

// consistency check
assert G*G eq ScalarMatrix(25,1);


BB0:=Matrix(B0);
Id25:= ScalarMatrix(25,1);
Id19:=ScalarMatrix(19,1);
N0:=Matrix(Solution(BB0,A));

assert N0*BB0 eq A;
assert M0*A eq BB0;
assert M0*N0 eq Id19;
assert N0*Gram0*Transpose(N0) eq Matrix(A);

G19:=M0*G*N0;
assert Order(G19) eq 2;
Gal19:=MatrixGroup<19, Integers() |[G19]>;

G192:=ChangeRing(G19,F2);
Gal192:=MatrixGroup<19,F2|[G192]>;

// We check that there are no vectors with a one-dimensional orbit under the action of the Galois group

VLambda2:=[v : v in Lambda2 | Dimension(sub<V2 | Orbit(Gal192,v)>) le 1];

assert VLambda2 eq [Zero(V2)];


// End of Proof of Theorem 3.8
print "Theorem 3.8 is confirmed.";
print "**************************";
print "";

// Proof of Proposition 3.10
print "Verifying Proposition 3.10.";

gmpic:=GModule(Gal19);
cmpic:=CohomologyModule(Gal19,gmpic);
cgpic0:=CohomologyGroup(cmpic,0);
cgpic1:=CohomologyGroup(cmpic,1);
cgpic2:=CohomologyGroup(cmpic,2);
print "The group H^0(G, Pic bar(S)) is isomorphic to the ", cgpic0; 
print "*********";
print " ";
print "The group H^1(G, Pic bar(S)) is isomorphic to the ", cgpic1; 
print "*********";
print " ";
print "The group H^2(G, Pic bar(S)) is isomorphic to the ", cgpic2; 
print "*********";
print " ";

// End of Proof of Proposition 3.10
print "Proposition 3.10 is confirmed.";
print "**************************";
print "";




