/*******************************************************/
/* 						                               */
/*           Magma code accompanying Section 5         */ 
/*              				                       */
/*                    by Bartosz Naskręcki             */
/*						                               */
/*******************************************************/


//Section 5 code
//*****************
R<a,b,c,d,A>:=PolynomialRing(Rationals(),5);
eq1:=[1411985089 - 631459755*A + 18*a*c, 131587540863282 - 58847737271814*A + 108*c^3 + 729*d^2, 
-238992218766044 + 106880569389324*A - 1458*b*d, 
 131587540863282 + 108*a^3 - 58847737271814*A + 729*b^2,A^2-5];
P:=Scheme(AffineSpace(R),eq1);
//this the scheme that parametrises pairs of elliptic curves
//E1:y^2=x^3+a*x+b
//E2:y^2=x^3+c*x+d
//which form the Shioda-Inose structure on the X_{DY}

//over field K24 we have K24-rational points on S
K24:=NumberField(Polynomial([1, 0, 0, 0, 0, 0, 24, 0, 0, 0, 0, 0, -18, 0, 0, 0, 0, 0, -24, 0, 0, 0, 0, 0, 1]));

K4<sq2,sq5>:=NumberField([Polynomial([-2,0,1]),Polynomial([-5,0,1])]);//QQ(sqrt(2),sqrt(5))
K8rel<alpha>:=ext<K4|Polynomial([-(sq5+1)/2,0,1])>;
K24rel<beta>:=ext<K8rel|Polynomial([-(sq2-1),0,0,1])>;
assert IsIsomorphic(K24,K24rel);
print "Remark 5.3 is confirmed.";
print "*******************************************";
print "";

print "Computing points on the scheme P over K24 = ",K24;
ptsK24:=RationalPoints(P,K24);
assert #ptsK24 eq 4;
print "Scheme P has 4 points over K24";
print "*******************************************";
print "";


Eab:=EllipticCurve([ptsK24[1][1],ptsK24[1][2]]);
Ecd:=EllipticCurve([ptsK24[1][3],ptsK24[1][4]]);

sq5:=Sqrt(K24!5);
sq2:=Sqrt(K24!2);
E11:=EllipticCurve([0,4,0,2*(1 - 4*sq2 - 3*sq5),0]);
Em11:=EllipticCurve([0,-4,0,2*(1 + 4*sq2 - 3*sq5),0]);
assert IsQuadraticTwist(Eab,E11);
assert IsQuadraticTwist(Ecd,Em11);

print "Elliptic curve E11 is isomorphic to Eab over Qbar";
print "Elliptic curve Em11 is isomorphic to Ecd over Qbar";
print "*******************************************";
print "";

//Proof of Prop. 5.4
K4<sq2,sq5>:=NumberField([Polynomial([-2,0,1]),Polynomial([-5,0,1])]);//QQ(sqrt(2),sqrt(5))
E11:=EllipticCurve([0,4,0,2*(1 - 4*sq2 - 3*sq5),0]);
Em11:=EllipticCurve([0,-4,0,2*(1 + 4*sq2 - 3*sq5),0]);
assert IsQuadraticTwist(Eab,E11);
assert IsQuadraticTwist(Ecd,Em11);

print "Elliptic curve E11 is isomorphic to Eab over Qbar";
print "Elliptic curve Em11 is isomorphic to Ecd over Qbar";
print "*******************************************";
print "";

//Proof of Prop. 5.4
K<eta>:=ext<K4|Polynomial([(-37*sq5 - 117)*sq2 - 74*sq5 - 117,0,1])>;
F<t>:=FunctionField(K);
//Inose fibration model of our Kummer surface
EIno:=EllipticCurve([((-71 - 45*sq5)*t^4)/6, ((3 - sq5)*t^4)/2 + ((-551 - 189*sq5)*t^6)/27 + ((3 - sq5)*t^8)/2]); 


EInosepar:=func<a,b,c,d,t|EllipticCurve([-((a^2 - 3*b)*(c^2 - 3*d)*t^4)/3,(t^4*(27*a^2*b^2 - 108*b^3 + 8*a^3*c^3*t^2 - 36*a*b*c^3*t^2 - 36*a^3*c*d*t^2 +  162*a*b*c*d*t^2 + 27*c^2*d^2*t^4 - 108*d^3*t^4))/108])>;  
//this is an elliptic model for (x^3+a*x^2+b*x)-t^2*(y^3+c*y^2+d*y)

ee1:=EInosepar(aInvariants(E11)[2],aInvariants(E11)[4],aInvariants(Em11)[2],aInvariants(Em11)[4],t*eta);
ee2:=EIno;

assert QuadraticTwist(ee1,1/(2*eta)^2) eq ee2;
print "Proposition 5.4 is confirmed.";
print "*******************************************";
print "";

R<x> := PolynomialRing(Rationals()); 
K4<a> := NumberField(R![4, 0, -6, 0, 1]); //=Q(sqrt(2),sqrt(5))
s2:=Sqrt(K4!2);
s5:=Sqrt(K4!5);
E11small:=EllipticCurve([0,4,0,2*(1 - 4*s2 - 3*s5),0]);

//elliptic curve LMFDB 4.4.1600.1-256.1-i2 (http://www.lmfdb.org/EllipticCurve/4.4.1600.1/256.1/i/2)
E256i2:=ChangeRing(EllipticCurve([0, -1/2*a^3 + 2*a - 1, 0, -2*a^3 - 9/2*a^2 + 2*a + 4, 9/2*a^3 + 21/2*a^2 - 3*a - 8]),K4); 
LE11:=LSeries(E11small);
LE256i2:=LSeries(E256i2);

assert IsIsomorphic(E256i2,QuadraticTwist(E11small,1/2+1/s2));
print "The curve $E_{256.1-i2}$ is the quadratic twist of $E_{1,1}$ by the element $\frac{1}{2}+\frac{1}{sqrt{2}}$.";

KK4:=NumberField(Polynomial([-4,0,4,0,1])); //minimal field of definition of the element sqrt(1/2+1/sqrt(2))
artrepKK4:=ArtinRepresentations(KK4); //has a unique 2-dim Artin representation of D4
rho:=artrepKK4[5];//the unique 2-dim Artin representation of KK4
Lrho:=LSeries(rho);
TensorL:=TensorProduct(LE256i2,Lrho);
print "Verifying Prop.5.5...";
assert &and[Sqrt(EulerFactor(TensorL,NthPrime(i))) eq EulerFactor(LE11,NthPrime(i)): i in [1..100]];
print "Numerical verification of Prop. 5.5 confirmed up to primes p<=541.";
print "*******************************************";
print "";

//Remark 5.6 (Description of the isogeny class)
R<T>:=PolynomialRing(Rationals());
K<sq2,sq5>:=NumberField([T^2-2,T^2-5]);
E1:=EllipticCurve([0,4,0,2*(1 - 4*sq2 - 3*sq5),0]);
E2:=EllipticCurve([0,-4,0,2*(1 + 4*sq2 - 3*sq5),0]);
Ef:=func<e1,e2|EllipticCurve([0,4,0,2*(1 - e1*4*sq2 - e2*3*sq5),0])>;
IsTwist,tw:=IsQuadraticTwist(Ef(-1,1),E2);
assert IsTwist;
assert tw eq -1;

//isogeny phi:E1->E2 of degree 3
_,phi:=IsogenyFromKernel(E1,Factorization(DivisionPolynomial(E1,3))[1][1]);
assert IsIsomorphic(Codomain(phi),QuadraticTwist(Ef(-1,1),-1));
assert Degree(phi) eq 3;

//isogeny psi:E1->F of degree 2
T2,mT2:=TwoTorsionSubgroup(E1);
assert Order(T2) eq 2;
psi:=TwoIsogeny(mT2(T2.1));
F:=Codomain(psi);
assert IsIsomorphic(F,QuadraticTwist(Ef(-1,-1),-2));

//isogeny psiprim:E2->G of degree 2
TT2,mTT2:=TwoTorsionSubgroup(Codomain(phi));
psiprim:=TwoIsogeny(mTT2(TT2.1));
assert Order(TT2) eq 2;
G:=Codomain(psiprim);
assert IsIsomorphic(G,QuadraticTwist(Ef(1,-1),2));


Em1m1twm2:=QuadraticTwist(Ef(-1,-1),-2);
//bottom front-face isogeny of degree 3
_,botfrontisog:=IsogenyFromKernel(Em1m1twm2,Factorization(DivisionPolynomial(Em1m1twm2,3))[1][1]);
assert IsIsomorphic(Codomain(botfrontisog),QuadraticTwist(Ef(1,-1),2));
assert Degree(botfrontisog) eq 3;

factE1_7:=Factorization(DivisionPolynomial(E1,7));
//E1 to Etilde isogeny of degree 7
_,isog7:=IsogenyFromKernel(E1,factE1_7[1][1]*factE1_7[2][1]*factE1_7[3][1]);
Etilde:=Codomain(isog7);
assert Degree(isog7) eq 7;

factEm1m1twm2_7:=Factorization(DivisionPolynomial(Em1m1twm2,7));
//E_{-1,-1}^{(-2)} to star isogeny of degree 7
_,isog_downleft:=IsogenyFromKernel(Em1m1twm2,factEm1m1twm2_7[1][1]*factEm1m1twm2_7[2][1]*factEm1m1twm2_7[3][1]);
assert Degree(isog_downleft) eq 7;

E1m1tw2:=QuadraticTwist(Ef(1,-1),2);
factE1m1tw2_7:=Factorization(DivisionPolynomial(E1m1tw2,7));
//E_{1,-1}^{(2)} to star isogeny of degree 7
_,isog_downright:=IsogenyFromKernel(E1m1tw2,factE1m1tw2_7[1][1]*factE1m1tw2_7[2][1]*factE1m1tw2_7[3][1]);
assert Degree(isog_downright) eq 7;

Em11twm1:=QuadraticTwist(Ef(-1,1),-1);
factEm11twm1_7:=Factorization(DivisionPolynomial(Em11twm1,7));
//E_{-1,1}^{(-1)} to star isogeny of degree 7
_,isog_topright:=IsogenyFromKernel(Em11twm1,factEm11twm1_7[1][1]*factEm11twm1_7[2][1]*factEm11twm1_7[3][1]);
assert Degree(isog_topright) eq 7;

//in the notation of Figure 2
//star denoted elliptic curves (with respect to their possition)
Estarbotbackleft:=Codomain(isog_downleft);
Estarbotbackright:=Codomain(isog_downright);
Estartopbackright:=Codomain(isog_topright);

//checking that for the we can connect the vertices from Figure 2 with isogenies of degrees 2,3 and 7 in the cubical structure
Tsbbl,mTsbbl:=TorsionSubgroup(Estarbotbackleft);
assert Order(Tsbbl) eq 2;
isog_leftupback:=TwoIsogeny(mTsbbl(Tsbbl.1));
assert IsIsomorphic(Codomain(isog_leftupback),Etilde);

_,isog_backdowndeg3:=IsogenyFromKernel(Estarbotbackleft,Factorization(DivisionPolynomial(Estarbotbackleft,3))[1][1]);
assert Degree(isog_backdowndeg3) eq 3;
assert IsIsomorphic(Estarbotbackright,Codomain(isog_backdowndeg3));

_,isog_backtopdeg3:=IsogenyFromKernel(Etilde,Factorization(DivisionPolynomial(Etilde,3))[1][1]);
assert Degree(isog_backtopdeg3) eq 3;
assert IsIsomorphic(Estartopbackright,Codomain(isog_backtopdeg3));

Tstbr,mTstbr:=TorsionSubgroup(Estartopbackright);
assert Order(Tstbr) eq 2;
isog_rightupback:=TwoIsogeny(mTstbr(Tstbr.1));
assert IsIsomorphic(Estarbotbackright,Codomain(isog_rightupback));


assert Ef(1,1) eq E1;
assert IsIsomorphic(Ef(-1,1),QuadraticTwist(E2,-1));
assert IsIsomorphic(Ef(1,-1),QuadraticTwist(G,2));
assert IsIsomorphic(Ef(-1,-1),QuadraticTwist(F,-2));
print "Checked all the statements from Remark 5.6";
print "*******************************************";
print "";

//Lemma 5.7 irreducibility of the 5-division polynomial

R<x> := PolynomialRing(Rationals()); 
K4<a> := NumberField(R![4, 0, -6, 0, 1]); //=Q(sqrt(2),sqrt(5))
K8:=ext<K4|Factorization(CyclotomicPolynomial(5),K4)[1][1]>;
E256i2:=ChangeRing(EllipticCurve([0, -1/2*a^3 + 2*a - 1, 0, -2*a^3 - 9/2*a^2 + 2*a + 4, 9/2*a^3 + 21/2*a^2 - 3*a - 8]),K8); 
assert IsIrreducible(DivisionPolynomial(E256i2,5));    
print "Checked Lemma 5.7 irreducibility.";
print "*******************************************";
print "";

//verification of Thm.5.7
F<t>:=FunctionField(Rationals());
R<x>:=PolynomialRing(F);

Efib2:=EllipticCurve(16*t^5*(-1 + t + t^2)*x - (-1 - 2*t^2 + 8*t^3 + 3*t^4)*x^2 + x^3);//elliptic fibration from Prop.4.12
//this is used to provide the number of points on S

function PointCountOnS(q)
	Ered:=ChangeRing(Efib2,FunctionField(GF(q)));
	return NumberOfPointsOnSurface(Ered,1);
end function;

K2<sq2>:=QuadraticField(2);
RR<x2>:=PolynomialRing(K2);
K25<sq5>:=ext<K2|x2^2-5>;
K25abs:=AbsoluteField(K25);
_,iso:=IsIsomorphic(K25,K25abs);
E11:=EllipticCurve([0,4,0,K25abs!(2*(1 - 4*sq2 - 3*sq5)),0]);
M25:=MaximalOrder(K25);
prbad:=[x[1]: x in Factorization(Norm(Conductor(E11)))];
assert prbad eq [2];


//computes the F_p^2 trace of the reduction mod p of E11
function E11Trace(p)
pr:=Decomposition(M25!p)[1][1];
k,mk:=ResidueClassField(pr);
if #k eq p^2 then
E11k:=EllipticCurve([mk(x): x in aInvariants(E11)]);
return #E11k-(#k+1);
else
E11k:=EllipticCurve([mk(x): x in aInvariants(E11)]);
E11kbc:=ChangeRing(E11k,GF(#k^2));
return #E11kbc-(#k^2+1);
end if;
end function;

function TraceFormulaVer2(p)//checking the trace formula for the number of points over F_{p^2}
return PointCountOnS(p^2)-(1+p^4+18*p^2+E11Trace(p)^2);
end function;

print "Veryfing the trace formula over F_{p^2}...";
assert &and[IsZero(TraceFormulaVer2(NthPrime(k))): k in [2..20]|not NthPrime(k) in prbad];
print "Numerical verification of the trace formula for S(F_{p^2}) for primes p<=71.";
print "*******************************************";
print "";

//numerical verification of the point count formula over F_{p} in Thm.5.7

function H(p,L,K)  //H(p) function
	 pol:=EulerFactor(L,p);
	 
	 dec:=Decomposition(K,p);
	 _,sqpol:=IsPower(pol,2);
	 
	 if #dec eq 4 then
		 ap:=Coefficients(Factorization(sqpol)[1][1])[2];
		 print ap;
		 return ap^2-p;
	 end if;
	 
	 if #dec eq 2 and Factorization(sqpol)[1][2] eq 1 then
		 ap:=Coefficients(sqpol)[3];
		 print ap;
		 return -ap+LegendreSymbol(10,p)*p;
	 end if;

	 if #dec eq 2 and Factorization(sqpol)[1][2] eq 2 then
		 print 0;
		 return -LegendreSymbol(10,p)*p;
	 end if;
	 
	 print "problem";
	 assert false;
	 
 end function;
 
 K4<a> := NumberField(Polynomial([4, 0, -6, 0, 1]));//=Q(sqrt(2),sqrt(5))
 E256i2:=EllipticCurve([0, -1/2*a^3 + 2*a - 1, 0, -2*a^3 - 9/2*a^2 + 2*a + 4, 9/2*a^3 + 21/2*a^2 - 3*a - 8]); 
 LE256i2:=LSeries(E256i2);
 
 function TraceTotal(p)
 return (1+p^2+18*p+LegendreSymbol(5,p)*p)+H(p,LE256i2,K4);
 end function;
 
 //numerical verification up to p<=1583
 print "Veryfing the trace formula over F_{p}...";
 assert &and[TraceTotal(NthPrime(i)) eq PointCountOnS(NthPrime(i)): i in [4..250]];
 print "Trace formula over F_{p} is correct for primes p<=1583.";
 print "*******************************************";
 print "";
 
 //classical weight 2 newform trace formula
 
 C:=CuspForms(Gamma1(160),2);
 new:=NewSubspace(C);
 newf:=Newforms(new);
 //// Make newform 160.2.f.a in Magma
 ff:=[x: x in newf|#x eq 4 and IsIsomorphic(BaseField(x[1]),MergeFields(QuadraticField(2),QuadraticField(-3))[1])][1];   
 function TraceTotalvariant(p)
 	return (1+p^2+18*p+LegendreSymbol(5,p)*p)+Coefficient(ff[1],p)^2-LegendreSymbol(10,p)*p eq PointCountOnS(p);
 end function;
 
 assert  &and[TraceTotalvariant(NthPrime(k)): k in [4..1000]];

 //Section 5.2: supersingular reduction
 
 K2<sq2>:=QuadraticField(2);
 RR<x2>:=PolynomialRing(K2);
 K25<sq5>:=ext<K2|x2^2-5>;
 K25abs:=AbsoluteField(K25);
 _,iso:=IsIsomorphic(K25,K25abs)
 E11:=EllipticCurve([0,4,0,K25abs!(2*(1 - 4*sq2 - 3*sq5)),0]);
 pol:=AbsoluteMinimalPolynomial(jInvariant(E11));
 R<T>:=PolynomialRing(Integers());
 print "P(T)=",R!pol;
 super:=func<p|Resultant(pol,SupersingularPolynomial(p))>;
 //if super(p) eq 0; then we have a supersingular prime
 print "Producing a list of primes of supersingular reduction...";
 print "Primes of supersingular reduction for E_{1,1} up to p<=104729: ",[NthPrime(k): k in [5..10000]|super(NthPrime(k)) eq 0];
 print "*******************************************";
 print "";
 
 //End of code for Section 5
 //*************************
 
 print "Numerical verification of Section 5 is complete.";
 print "*******************************************";
 print "";
 



