/*******************************************************/
/* 						                               */
/*           Magma code accompanying Section 6         */ 
/*              				                       */
/*                    by Bartosz Naskręcki             */
/*						                               */
/*******************************************************/

//Section 6 code
//*****************

print "Verifying Section 6.1.1";
//points computation & discriminant formula 
F<t>:=FunctionField(Rationals());
E2:=EllipticCurve([0, 1 + 2*t^2 - 8*t^3 - 3*t^4, 0, 16*t^5*(-1 + t + t^2), 0]);
Loc:=LocalInformation(E2);
//is in the globally minimal model
assert &and[x[2] lt 12: x in FactorizationOfQuotient(Discriminant(E2))|Degree(x[1]) gt 0] and ((24-Degree(Numerator(Discriminant(E2)))) lt 12);
print "E2 is in the globally minimal model.";
//computation of the Picard rank
rho_triv:=2+&+[Degree(x[1])*(NumberOfComponents(x[5])-1): x in Loc];//contribution to Picard rank from the trivial sublattice

//contribution from the Mordell-Weil group
P3:=E2![4*t^3,4*t^3*(t^2-1)];
assert Height(P3) gt 0;//hence a point is of infinite order
print "Point P3", P3,"has infinite order on ", E2;

//we obtain the following lower bound for Picard rank
print "Picard rank of S>=",rho_triv+1;

//finding primes of good reduction for the elliptic surface E2
//we look for a prime number p such that in the globally minimal model for E2, the squarefree part of each coefficient a_i, of the discriminant D,
//of the numerator and the denominator of j are separable under reduction modulo p
//this guarantees that performing the Tate algorithm to resolve singularities will end up with the surface of the same time (aka. we can provide a simultaneous resolution in char. 0 and char.p)

sqdisc:=func<pol|Discriminant(SquarefreePart(Numerator(pol)))>;

F<t>:=FunctionField(Integers());
E2:=EllipticCurve([0, 1 + 2*t^2 - 8*t^3 - 3*t^4, 0, 16*t^5*(-1 + t + t^2), 0]);

a2:=Numerator(aInvariants(E2)[2]);
a4:=Numerator(aInvariants(E2)[4]);
assert &and[aInvariants(E2)[i] eq 0: i in [1,3,5]];
disc:=Numerator(Discriminant(E2));
assert Degree(Denominator(Discriminant(E2))) eq 0;
jnum:=Numerator(jInvariant(E2));
jden:=Denominator(jInvariant(E2));

badprimes:=[x[1]: x in Factorization(&*[sqdisc(pol): pol in [a2,a4,disc,jnum,jden]])];//this set is an overset of primes of bad reduction
print "If p is a prime of bad reduction for the surface E2, then p belongs to the set ", badprimes;
//upper bound on the Picard number from good reduction at two primes

assert not (31 in badprimes);
assert not (71 in badprimes);

function LFunctionComp(p)
	F<t>:=FunctionField(GF(p));
	E2:=EllipticCurve([0, 1 + 2*t^2 - 8*t^3 - 3*t^4, 0, 16*t^5*(-1 + t + t^2), 0]);
	P1:=E2![4*t^3,4*t^3*(t^2-1)];
	lfun:=LFunction(E2,[P1]:Check:=false);
	return lfun;
end function;

print "Computing the characteristic polynomial of Frobenius on H^2_{et](S_Fpbar,Q_ell)} for primes p=31,37";
assert Trace(FrobeniusActionOnTrivialLattice(ChangeRing(E2,FunctionField(GF(31))))) eq 18;
assert Trace(FrobeniusActionOnTrivialLattice(ChangeRing(E2,FunctionField(GF(71))))) eq 18;
assert Numerator(F!Evaluate(LFunction(ChangeRing(E2,FunctionField(GF(31)))),1/t)) eq (t-31)^2*(t^2 + 46*t + 961);
assert Numerator(F!Evaluate(LFunction(ChangeRing(E2,FunctionField(GF(71)))),1/t)) eq (t-71)^2*(t^2 - 2*t + 5041);
print "For p=31 we have (t-31)^20*(t^2 + 46*t + 961)";
print "For p=71 we have (t-71)^20*(t^2 - 2*t + 5041)";

print "*********************************";
print "";
print "Factorization of the Frobenius characteristic polynomial on the transcendental part of H^2 cohomology.";
print "For p=31: ",Factorization(Numerator(Evaluate(t^2 + 46*t + 961,t*31)))[2][1];
print "For p=71: ",Factorization(Numerator(Evaluate(t^2 - 2*t + 5041,t*71)))[2][1];
//hence we conclude that the roots of the scaled polynomials are not roots of unit, since the defining polynomials have a non-trivial leading coefficient
//We conclude from that the upper bound for the Picard rank to be 20

//we use both primes to compute that the rank 20 is not possible in char. 0 since otherwise the values of the discriminant of the Neron-Severi group would not differ by a square

assert not IsSquare((Evaluate(t^2 + 46*t + 961,31)/31)/(Evaluate(t^2 - 2*t + 5041,71)/71));
//thus we conclude that the Picard rank equals 19
print "Picard rank of S<=19.";

print "*******************";
print "";

//Section 6.1.2 and Section 6.1.3
print "Verifying Section 6.1.2 and Section 6.1.3";
//free generator

ss:=Sort([<4-(a*(10-a)/10+b*(2-b)+c*(2-c)/2+d*(2-d)/2+e*(2-e)/2+f*(4-f)/4),<a,b,c,d,e,f>>:a in [0..9], b,c,d,e in [0..1], f in [0..3]]);
assert {x[1]: x in ss|x[1] gt 0 and x[1] lt 3/20} eq {1/10};

//hence since P2 has height 3/20 it is not equal to m*Q+T for any Q and a torsion point T, since otherwise there would exist a point of height 3/(m^2*20)
//but the minimal height is 1/10, so there is no such m possible.

//discriminant of the Neron-Severi lattice
//discriminant formula

KS:=KodairaSymbol;
assert [<Degree(x[1]),x[5]>: x in Loc] eq [ <2, KS("I1")>, <2, KS("I2")>, <1, KS("I4")>, <1, KS("I2")>, <1, KS("I10")>, <1, KS("I2")> ]; 
disc:=2*2*4*2*10*2*3/20/2^2;
assert disc eq 24;

m2map:=MultiplicationByMMap(E2,2);
Amb:=Domain(m2map);
FF<x,y,z>:=FieldOfFractions(Parent(Amb.1));
print "On E2 the formula for the x-coordinate of 2*(x,y) is ",  Evaluate(DefiningPolynomials(m2map)[1]/DefiningPolynomials(m2map)[3],[x,y,1]);

//with P3 and E2![0,0] we have a sublattice of NS of discriminant 24. If this would be not maximal, then possibly it would be not saturated at 2
//this is only possible when the torsion subgroup E1(Qbar(t)) would not be equal to Z/2 and would contain more 2-power torsion points
//Suppose we would have a point of order 2^r, then the discriminant formula tells us, that discNS | 24/(2^(2r)) and since disc NS is an integer
//it must be r=1, hence we have found a full torsion subgroup

print "Neron-Severi lattice has discriminant equal to ",disc;

//points of intersection with components of the reducible fibres

F<t>:=FunctionField(Rationals());
E2:=EllipticCurve([0, 1 + 2*t^2 - 8*t^3 - 3*t^4, 0, 16*t^5*(-1 + t + t^2), 0]);
Loc:=LocalInformation(E2);

function EschemeAtT(t) //scheme E2_t at a specific value of t
	P2<x,y,z>:=ProjectiveSpace(FieldOfFractions(Parent(t)),2);	
	return Scheme(P2,[-x^3 + (3*t^4 + 8*t^3 - 2*t^2 - 1)*x^2*z + y^2*z + (-16*t^7 - 16*t^6 + 16*t^5)*x*z^2]);
end function;

function EschemeAtTinf(t) //scheme E2_(1/t) at a specific value of t; allows us to cover the place at infinity
	P2<x,y,z>:=ProjectiveSpace(FieldOfFractions(Parent(t)),2);	
	return Scheme(P2,[-x^3 - (-3 - 8*t + 2*t^2 + t^4)*x^2*z + y^2*z + 16*t*(-1 - t + t^2)*x*z^2]);
end function;

Egen:=Curve(EschemeAtT(t));
assert IsIsomorphic(EllipticCurve(Egen,Egen![0,1,0]),E2);
Egeninf:=Curve(EschemeAtTinf(t));
assert IsIsomorphic(EllipticCurve(Egeninf,Egeninf![0,1,0]),EllipticCurve([Evaluate(a,1/t): a in aInvariants(E2)]));

K2<s>:=SplittingField([x[1]: x in Factorization(Numerator(Discriminant(E2)))|x[2] gt 1]);
roo:=[x[1]: x in Roots(Numerator(Discriminant(E2)),K2)];

function SingularPointOnE2atT(t)
	return Eltseq(RationalPoints(SingularSubscheme(EschemeAtT(t)))[1])[1..2];
end function;

//components intersection for the point P3
P3inf:=[Evaluate(coord,0): coord in [t, -4*(-1 + t)*t*(1 + t)]];
assert P3inf eq Eltseq(RationalPoints(SingularSubscheme(EschemeAtTinf(0)))[1])[1..2];
print "Point P3 intersects the non-trivial components at t=oo.";

P3eval:=func<t|[4*t^3,4*t^3*(t^2-1)]>;

for t0 in roo do
	if P3eval(t0) eq SingularPointOnE2atT(t0) then
		print "Point P3 intersects a non-trivial component at t=",t0;
	end if;
end for;

//components intersection for the two-torsion point (0,0)
Tors:=[0,0];
assert Tors eq Eltseq(RationalPoints(SingularSubscheme(EschemeAtTinf(0)))[1])[1..2];
print "Point (0,0) intersects the non-trivial components at t=oo.";

for t0 in roo do
	if Tors eq SingularPointOnE2atT(t0) then
		print "Point (0,0) intersects a non-trivial component at t=",t0;
	end if;
end for;
print "Minimal polynomial of s = ",MinimalPolynomial(s);

//each reducible fibre has a structure of an n-gon with a cyclic group structure it follows that
//the component number of intersection for the point Tors=(0,0) is
//at t=0 (type I_10): 5-th component

//for the point P3
//at t=-1:(type I_4):

//from the Lemma 5.1 in Silverman, J., "Computing heights on elliptic curves", Math. Comp. 51 (1988), no. 183, 339–358.
//it follows that 
//at t=0: P3 intersects the third component, (0,0) the fifth
//at t=-1: P3 intersects the first component, (0,0) the second
print "At t=0: P3 intersects the third component, (0,0) the fifth.";
print "At t=-1: P3 intersects the first component, (0,0) the second.";
//from the Shioda height formula it follows that

assert Height(P3) eq 2*2-21/10-1/2-1/2-3/4; 
assert Height(E2![0,0]) eq 2*2-25/10-1/2-1/2-1/2;

print "*******************";
print "";
//Section 6.1.4
print "Verifying Section Section 6.1.4.";
//ordered generators: tinf, P, T, tm1, tsq51, tsq52, b1, b2, b3, a1, a2, a3, a4, a5, a6,a7, a8, a9, F, O
tinf:=1;
P:=2;
T:=3;
tm1:=4;
tsq51:=5;
tsq52:=6;
b1,b2,b3:=Explode([7,8,9]);
a1,a2,a3,a4,a5,a6,a7,a8,a9:=Explode([10..18]);
F:=19;
O:=20;
S := {@x: x in [1..20] @};
GraphIntersections, V, E := Graph< S | {{tinf, P}, {tinf, T}, {tm1, P}, {tsq51, T}, {tsq52, T}, {b1, b2}, {b2, b3}, {P, b1}, {a1, a2}, {a2, a3}, {a3, a4}, {a4, a5}, {a5,a6}, {a6, a7}, {a7, a8}, {a8, a9}, {P, a3}, {T, a5}, {P, F}, {T, F}, {F, O}} >;
   
GramMatrix:=AdjacencyMatrix(GraphIntersections)+(-2)*IdentityMatrix(Integers(),20);
GramMatrix[19,19]:=0; //F^2=0

//intersection pairing matrix for the following components of the fibration
//from Prop.4.12


M1:=Matrix(Integers(),20,[-2, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,0, 0, 1, -2, \
0, 1, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, -2, 0, 1, \
1, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 1, 0, -2, 0, 0, 0, 0, \
0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, -2, 0, 0, 0, 0, 0, 0, \
0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, -2, 0, 0, 0, 0, 0, 0, 0, 0, \
0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, -2, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, \
0, 0, 0, 0, 0, 0, 0, 0, 0, 1, -2, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, \
0, 0, 0, 0, 0, 0, 0, 1, -2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, \
0, 0, 0, 0, 0, 0, -2, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, \
0, 0, 0, 1, -2, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, \
0, 1, -2, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, \
-2, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, -2, \
1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, -2, 1, 0, \
0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, -2, 1, 0, 0, 0, \
0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, -2, 1, 0, 0, 0, 0, 0, \
0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, -2, 0, 0, 0, 1, 1, 0, 0, 0, \
0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, \
0, 0, 0, 0, 0, 0, 0, 0, 0, 1, -2]);

assert GramMatrix eq M1;
print "Intersection pairing matrix";
print "Generators order: tinf, P, T, tm1, tsq51, tsq52, b1, b2, b3, a1, a2, a3, a4, a5, a6,a7, a8, a9, F, O";
print GramMatrix;

assert Determinant(M1) eq 0;
assert Dimension(Kernel(M1)) eq 1;
print "Linear relation between the generators",Basis(Kernel(M1));

//submatrix of dimension 19 which skips the entries for tinf (not included in the basis of the lattice)
Mbas:=Submatrix(M1,2,2,19,19);
L:=LatticeWithGram(Mbas:CheckPositive:=false);
assert Determinant(L) eq 24;
bas:=Basis(L);
P:=bas[1];
T:=bas[2]; 
F:=bas[18];
O:=bas[19];
tup:=[P-O-2*F,T-O-2*F] cat bas[3..17] cat [F,F+O];
MM:=Matrix(Integers(),#tup,&cat [[InnerProduct(tup[i],tup[j]): i in [1..#tup]]: j in [1..#tup]]);
assert Determinant(LatticeWithGram(MM:CheckPositive:=false)) eq 24;

tupess:=[P-O-2*F,T-O-2*F] cat bas[3..17];
Mess:=Matrix(Integers(),#tupess,&cat [[InnerProduct(tupess[i],tupess[j]): i in [1..#tupess]]: j in [1..#tupess]]);

EssLat:=LatticeWithGram(-Mess);//is positive definite
assert Determinant(EssLat) eq 24;
